#ifndef UART_H
#define UART_H

#include "abstract-uart.hpp"
#include "uart-gpio.hpp"
#include "uart-register.hpp"

namespace stm32f072xb {

template <UartRegister T, Port V>
class Uart : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<T, UartGpioType::TX, V> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<T, UartGpioType::RX, V> rxGpio);
	Uart(AbstractUartInit uartInit, UartGpio<T, UartGpioType::TX, V> txGpio, UartGpio<T, UartGpioType::RX, V> rxGpio);

   private:
	UartGpio<T, UartGpioType::TX, V> _txGpio;
	UartGpio<T, UartGpioType::RX, V> _rxGpio;
};

template <>
class Uart<UartRegister::UART1, Port::A> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART1, UartGpioType::TX, Port::A> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART1, UartGpioType::RX, Port::A> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART1, UartGpioType::TX, Port::A> txGpio,
	     UartGpio<UartRegister::UART1, UartGpioType::RX, Port::A> rxGpio);

   private:
	UartGpio<UartRegister::UART1, UartGpioType::TX, Port::A> _txGpio;
	UartGpio<UartRegister::UART1, UartGpioType::RX, Port::A> _rxGpio;
};

template <>
class Uart<UartRegister::UART1, Port::B> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART1, UartGpioType::TX, Port::B> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART1, UartGpioType::RX, Port::B> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART1, UartGpioType::TX, Port::B> txGpio,
	     UartGpio<UartRegister::UART1, UartGpioType::RX, Port::B> rxGpio);

   private:
	UartGpio<UartRegister::UART1, UartGpioType::TX, Port::B> _txGpio;
	UartGpio<UartRegister::UART1, UartGpioType::RX, Port::B> _rxGpio;
};

template <>
class Uart<UartRegister::UART2, Port::A> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> txGpio,
	     UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> rxGpio);

   private:
	UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> _txGpio;
	UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> _rxGpio;
};

template <>
class Uart<UartRegister::UART2, Port::D> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART2, UartGpioType::TX, Port::D> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART2, UartGpioType::RX, Port::D> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART2, UartGpioType::TX, Port::D> txGpio,
	     UartGpio<UartRegister::UART2, UartGpioType::RX, Port::D> rxGpio);

   private:
	UartGpio<UartRegister::UART2, UartGpioType::TX, Port::D> _txGpio;
	UartGpio<UartRegister::UART2, UartGpioType::RX, Port::D> _rxGpio;
};

template <>
class Uart<UartRegister::UART3, Port::B> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::TX, Port::B> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::RX, Port::B> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART3, UartGpioType::TX, Port::B> txGpio,
	     UartGpio<UartRegister::UART3, UartGpioType::RX, Port::B> rxGpio);

   private:
	UartGpio<UartRegister::UART3, UartGpioType::TX, Port::B> _txGpio;
	UartGpio<UartRegister::UART3, UartGpioType::RX, Port::B> _rxGpio;
};

template <>
class Uart<UartRegister::UART3, Port::C> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::TX, Port::C> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::RX, Port::C> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART3, UartGpioType::TX, Port::C> txGpio,
	     UartGpio<UartRegister::UART3, UartGpioType::RX, Port::C> rxGpio);

   private:
	UartGpio<UartRegister::UART3, UartGpioType::TX, Port::C> _txGpio;
	UartGpio<UartRegister::UART3, UartGpioType::RX, Port::C> _rxGpio;
};

template <>
class Uart<UartRegister::UART3, Port::D> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::TX, Port::D> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART3, UartGpioType::RX, Port::D> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART3, UartGpioType::TX, Port::D> txGpio,
	     UartGpio<UartRegister::UART3, UartGpioType::RX, Port::D> rxGpio);

   private:
	UartGpio<UartRegister::UART3, UartGpioType::TX, Port::D> _txGpio;
	UartGpio<UartRegister::UART3, UartGpioType::RX, Port::D> _rxGpio;
};

template <>
class Uart<UartRegister::UART4, Port::A> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART4, UartGpioType::TX, Port::A> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART4, UartGpioType::RX, Port::A> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART4, UartGpioType::TX, Port::A> txGpio,
	     UartGpio<UartRegister::UART4, UartGpioType::RX, Port::A> rxGpio);

   private:
	UartGpio<UartRegister::UART4, UartGpioType::TX, Port::A> _txGpio;
	UartGpio<UartRegister::UART4, UartGpioType::RX, Port::A> _rxGpio;
};

template <>
class Uart<UartRegister::UART4, Port::C> : public AbstractUart {
   public:
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART4, UartGpioType::TX, Port::C> txGpio);
	Uart(AbstractUartInit uartInit, UartGpio<UartRegister::UART4, UartGpioType::RX, Port::C> rxGpio);
	Uart(AbstractUartInit uartInit,
	     UartGpio<UartRegister::UART4, UartGpioType::TX, Port::C> txGpio,
	     UartGpio<UartRegister::UART4, UartGpioType::RX, Port::C> rxGpio);

   private:
	UartGpio<UartRegister::UART4, UartGpioType::TX, Port::C> _txGpio;
	UartGpio<UartRegister::UART4, UartGpioType::RX, Port::C> _rxGpio;
};

}  // namespace stm32f072xb
#endif
